// FesSelfMovement.js Ver.1.1.0
// MIT License (C) 2022 あわやまたな
// http://opensource.org/licenses/mit-license.php

/*:
* @target MV MZ
* @plugindesc ツクールフェスにあった単純な自律移動を再現します。
* @author あわやまたな (Awaya_Matana)
* @url https://awaya3ji.seesaa.net/
* @help
* 自律移動：カスタムに下記のスクリプトを入力することで新たな移動タイプを
* 使用可能になります。
*
* 【スクリプト（自律移動：カスタム）】
* this.setEscape();        //移動タイプを逃げるにします。
* this.setRecipHorz(向き); //移動タイプを横に往復にします。
* this.setRecipVert(向き); //移動タイプを縦に往復にします。
*
* 「逃げる」はプレイヤーから遠ざかる行動を自律移動仕様で実行します。
* 「横に往復」は直進→壁にぶつかる→左右反転→直進を繰り返します。
* 「縦に往復」は直進→壁にぶつかる→上下反転→直進を繰り返します。
*
* 【向き】
* 下：2　左：4　右：6　上：8
* 往復を開始する向きを強制します。
* 未入力だと現在の進行方向を維持。
*
* 【メモ】
* マップで初めて往復を開始した時、横に往復は左、縦に往復は下に動き始めます。
* これを変えたい場合はメモ欄に以下を入力してください。
*
* <fesRecip:6>   //横に往復を右から開始する。
* <fesRecip:8>   //縦に往復を上から開始する。
* <fesRecip:6,8> //横に往復を右、縦に往復を上から開始する。
* <fesRecip:8,6> //横に往復を右、縦に往復を上から開始する。
* <fesRecip>     //横に往復を右、縦に往復を上から開始する。
*
* ［更新履歴］
* 2022/02/19：Ver.1.0.0　公開。
* 2022/02/20：Ver.1.1.0　往復のウェイトを補正。
*
* @param setMoveTypeId
* @text 移動タイプIDの設定
* @desc 使用する移動タイプIDを設定値～設定値+2まで予約します。
* 0～3はコアスクリプトで予約済みです。
* @type number
* @default 4
* @min 4
*
* @param simplifyRandom
* @text 自律移動：ランダムの簡素化
* @desc ランダムの偏りをなくします。
* @type boolean
* @default false
*
* @param simplifyApproach
* @text 自律移動：近づくの簡素化
* @desc 挙動の不確定要素を取り除きます。
* @type boolean
* @default false
*
* @param simplifyEscape
* @text 自律移動：逃げるの簡素化
* @desc 挙動の不確定要素を取り除きます。
* @type boolean
* @default false
*
*/

'use strict';
{
	const pluginName = document.currentScript.src.match(/^.*\/(.*).js$/)[1];
	const param = PluginManager.parameters(pluginName);
	const paramTypeId = Number(param['setMoveTypeId']);
	const simplifyRandom = param['simplifyRandom'] === 'true';
	const simplifyApproach = param['simplifyApproach'] === 'true';
	const simplifyEscape = param['simplifyEscape'] === 'true';

	const _Game_Event_initialize = Game_Event.prototype.initialize;
	Game_Event.prototype.initialize = function(mapId, eventId) {
		_Game_Event_initialize.call(this, mapId, eventId);
		this.fesRecipMeta();
	};

	Game_Event.prototype.fesRecipMeta = function() {
		const fesRecip = this.event().meta["fesRecip"];
		if (!fesRecip) return;
		const dir = typeof fesRecip === 'string' ? new Set(fesRecip.split(',').map(Number)) : 0;
		if (!dir || dir.has(6)) this._reciprocateHorz = 6;
		if (!dir || dir.has(8)) this._reciprocateVert = 8;
	};

	const _Game_Event_initMembers = Game_Event.prototype.initMembers;
	Game_Event.prototype.initMembers = function() {
		this._reciprocateHorz = 4;
		this._reciprocateVert = 2;
		_Game_Event_initMembers.call(this);
	};
	
	const TYPE_MOVE_AWAY = paramTypeId;
	const TYPE_RECIPROCATE_HORIZONTALLY = paramTypeId +1;
	const TYPE_RECIPROCATE_VERTICALLY = paramTypeId +2;

	Game_Event.prototype.setEscape = function() {
		this._moveType = TYPE_MOVE_AWAY;
		this.updateSelfMovement();
	};

	Game_Event.prototype.setRecipHorz = function(dir) {
		this._moveType = TYPE_RECIPROCATE_HORIZONTALLY;
		if (dir === 4 || dir === 6) this._reciprocateHorz = dir;
		this.updateSelfMovement();
	};

	Game_Event.prototype.setRecipVert = function(dir) {
		this._moveType = TYPE_RECIPROCATE_VERTICALLY;
		if (dir === 2 || dir === 8) this._reciprocateVert = dir;
		this.updateSelfMovement();
	};

	const _Game_Event_updateSelfMovement = Game_Event.prototype.updateSelfMovement;
	Game_Event.prototype.updateSelfMovement = function() {
		_Game_Event_updateSelfMovement.call(this);
		if (!this._locked && this.isNearTheScreen() && this.checkStop(this.stopCountThreshold())) {
			switch (this._moveType) {
				case TYPE_MOVE_AWAY:
					this.moveTypeAwayFromPlayer();
					break;
				case TYPE_RECIPROCATE_HORIZONTALLY:
					this.reciprocateHorizontally();
					break;
				case TYPE_RECIPROCATE_VERTICALLY:
					this.reciprocateVertically();
					break;
			}
		}
	};

	Game_Event.prototype.moveTypeAwayFromPlayer = function() {
		if (this.isNearThePlayer()) {
			switch (Math.randomInt(6)) {
				case 0:
				case 1:
				case 2:
				case 3:
					this.moveAwayFromPlayer();
					break;
				case 4:
					this.moveRandom();
					break;
				case 5:
					this.moveForward();
					break;
			}
		} else {
			this.moveRandom();
		}
	};

	Game_Event.prototype.reciprocateHorizontally = function() {
		const lastDir = this._reciprocateHorz;
		this.moveStraight(lastDir);
		if (!this.isMovementSucceeded()) {
			this._reciprocateHorz = this.reverseDir(lastDir);
			this._stopCount = this.stopCountThreshold() - (5 - this.moveFrequency());
		}
	};

	Game_Event.prototype.reciprocateVertically = function() {
		const lastDir = this._reciprocateVert;
		this.moveStraight(lastDir);
		if (!this.isMovementSucceeded()) {
			this._reciprocateVert = this.reverseDir(lastDir);
			this._stopCount = this.stopCountThreshold() - (5 - this.moveFrequency());
		}
	};

	if (simplifyRandom) {
		Game_Event.prototype.moveTypeRandom = function() {
			this.moveRandom();
		};
	}

	if (simplifyApproach) {
		Game_Event.prototype.moveTypeTowardPlayer = function() {
			if (this.isNearThePlayer()) {
				this.moveTowardPlayer();
			} else {
				this.moveRandom();
			}
		};
	}

	if (simplifyEscape) {
		Game_Event.prototype.moveTypeAwayFromPlayer = function() {
			if (this.isNearThePlayer()) {
				this.moveAwayFromPlayer();
			} else {
				this.moveRandom();
			}
		};
	}
}