//=============================================================================
// RPGツクールMZ - LL_OriginalSelectWindow.js v1.0.0
//-----------------------------------------------------------------------------
// ルルの教会 (Lulu's Church)
// https://nine-yusha.com/
//
// Secondary use of this plugin is not permitted.
//=============================================================================

/*:
 * @target MZ
 * @plugindesc オリジナルの選択画面を実装します。
 * @author ルルの教会
 * @url https://nine-yusha.com/plugin/
 *
 * @help LL_OriginalSelectWindow.js
 *
 * オリジナルの選択画面を実装します。
 *
 * プラグインコマンド:
 *   選択画面の表示: 選択画面を表示し、結果を変数で受け取ります。
 *
 * 利用規約:
 *   このプラグインの二次利用、二次配布は禁止させていただきます。
 *
 * 作者: ルルの教会
 * 作成日: 2022/10/26
 *
 * @command showSelect
 * @text 選択画面の表示
 * @desc 選択画面を表示し、結果を変数で受け取ります。
 *
 * @arg choices
 * @text 選択肢リスト
 * @desc 選択肢として表示するリストを定義します。
 * @default []
 * @type struct<choices>[]
 *
 * @arg variableId
 * @text 結果受け取り変数
 * @desc 選択結果を受け取る変数IDを指定します。
 * @type variable
 *
 * @arg cancelEnabled
 * @text キャンセル許可
 * @desc キャンセルされた場合は-1が結果として返却されます。
 * @default false
 * @type boolean
 *
 * @arg iniPosition
 * @text 初期カーソル位置
 * @desc カーソルの初期位置です。(0にすると選択なし)
 * 数値を入力するか変数で指定することもできます。
 * @default 1
 * @type combo
 * @option $gameVariables.value(1)   // 変数ID:1の値
 * @option 0
 * @option 1
 * @option 2
 * @option 3
 * @option 4
 * @option 5
 * @option 6
 *
 * @arg backgroundImage
 * @text 背景画像
 * @desc 背景画像を使用する場合は画像を選択してください。
 * @dir img/pictures
 * @type file
 * @require 1
 *
 * @arg commandWindowSettings
 * @text コマンドウィンドウの設定
 * @desc ※この項目は使用しません
 *
 * @arg commandWindowX
 * @text X座標
 * @desc コマンドウィンドウの表示位置(X)です。
 * @default 0
 * @min -9999
 * @max 9999
 * @type number
 * @parent commandWindowSettings
 *
 * @arg commandWindowY
 * @text Y座標
 * @desc コマンドウィンドウの表示位置(Y)です。
 * @default 52
 * @min -9999
 * @max 9999
 * @type number
 * @parent commandWindowSettings
 *
 * @arg commandWindowWidth
 * @text 幅
 * @desc コマンドウィンドウの横幅です。
 * @default 360
 * @min -9999
 * @max 9999
 * @type number
 * @parent commandWindowSettings
 *
 * @arg commandWindowHeight
 * @text 高さ
 * @desc コマンドウィンドウの高さです。
 * @default 464
 * @min -9999
 * @max 9999
 * @type number
 * @parent commandWindowSettings
 *
 * @arg helpWindowSettings
 * @text ヘルプウィンドウの設定
 * @desc ※この項目は使用しません
 *
 * @arg helpWindowX
 * @text X座標
 * @desc ヘルプウィンドウの表示位置(X)です。
 * @default 0
 * @min -9999
 * @max 9999
 * @type number
 * @parent helpWindowSettings
 *
 * @arg helpWindowY
 * @text Y座標
 * @desc ヘルプウィンドウの表示位置(Y)です。
 * @default 712
 * @min -9999
 * @max 9999
 * @type number
 * @parent helpWindowSettings
 *
 * @arg helpWindowWidth
 * @text 幅
 * @desc ヘルプウィンドウの横幅です。
 * @default 1272
 * @min -9999
 * @max 9999
 * @type number
 * @parent helpWindowSettings
 *
 * @arg helpWindowHeight
 * @text 高さ
 * @desc ヘルプウィンドウの高さです。
 * @default 240
 * @min -9999
 * @max 9999
 * @type number
 * @parent helpWindowSettings
 */

/*~struct~choices:
 *
 * @param label
 * @text 文字列
 * @desc 選択肢として表示する文字列です。
 * \V[n]、\N[n]、\P[n]、\Gの制御文字が使用できます。
 * @type string
 *
 * @param helptext
 * @text 説明文
 * @desc 選択されている時に表示する説明文です。
 * 各種制御文字が使用できます。
 * @type multiline_string
 *
 * @param thumbnailSettings
 * @text サムネイル画像
 * @desc ※この項目は使用しません
 *
 * @param helpImage
 * @text 画像ファイル
 * @desc 選択されている時に表示するサムネイル画像です。
 * @dir img/pictures
 * @type file
 * @require 1
 * @parent thumbnailSettings
 *
 * @param helpImageX
 * @text X座標
 * @desc サムネイル画像の表示位置(X)です。
 * @default 384
 * @min -9999
 * @max 9999
 * @type number
 * @parent thumbnailSettings
 *
 * @param helpImageY
 * @text Y座標
 * @desc サムネイル画像の表示位置(Y)です。
 * @default 60
 * @min -9999
 * @max 9999
 * @type number
 * @parent thumbnailSettings
 *
 * @param switchId
 * @text スイッチID
 * @desc 指定するとスイッチON時のみ選択可能になります。
 * @type switch
 *
 * @param disabledLabel
 * @text 無効化時の文字列
 * @desc 選択肢が無効化されている時に代わりに表示する文字列です。
 * 空白にすると無効化時は非表示になります。
 * @type string
 */

(() => {
	"use strict";
	const pluginName = "LL_OriginalSelectWindow";

	const parameters = PluginManager.parameters(pluginName);
	const paramJsonParse = function(key, value) {
		try {
			return JSON.parse(value);
		} catch(e) {
			return value;
		}
	};

	// 設定保存用変数定義
	let choiceLists = [];
	let variableId = 0;
	let cancelEnabled = false;
	let iniPosition = 1;
	let backgroundImage = "";
	let commandWindowX = 0;
    let commandWindowY = 0;
	let commandWindowWidth = 0;
    let commandWindowHeight = 0;
	let helpWindowX = 0;
    let helpWindowY = 0;
	let helpWindowWidth = 0;
    let helpWindowHeight = 0;


	PluginManager.registerCommand(pluginName, "showSelect", function(args) {
        // 戦闘中は実行不可
        if ($gameParty.inBattle()) return;
		// 設定を取得
		variableId = Number(args.variableId || 0);
		cancelEnabled = String(args.cancelEnabled) === "true" ? true : false;
		iniPosition = eval(args.iniPosition);
		backgroundImage = String(args.backgroundImage || "");
		commandWindowX = Number(args.commandWindowX || 0);
		commandWindowY = Number(args.commandWindowY || 0);
		commandWindowWidth = Number(args.commandWindowWidth || 0);
		commandWindowHeight = Number(args.commandWindowHeight || 0);
		helpWindowX = Number(args.helpWindowX || 0);
		helpWindowY = Number(args.helpWindowY || 0);
		helpWindowWidth = Number(args.helpWindowWidth || 0);
		helpWindowHeight = Number(args.helpWindowHeight || 0);
		// 選択肢リストを取得
		choiceLists = JSON.parse(JSON.stringify(args.choices, paramJsonParse));

		// 選択肢が空の場合、終了
        if (choiceLists.length === 0) {
            console.log(pluginName + ": 有効な選択肢が存在しなかったため、コマンドがスキップされました");
            return;
        }

        SceneManager.push(Scene_ExOriginalSelectWindow);
    });


	//-----------------------------------------------------------------------------
    // Scene_ExOriginalSelectWindow
    //
    // オリジナル選択ウィンドウの独自シーンを定義します。

	function Scene_ExOriginalSelectWindow() {
        this.initialize(...arguments);
    }

	Scene_ExOriginalSelectWindow.prototype = Object.create(Scene_MenuBase.prototype);
	Scene_ExOriginalSelectWindow.prototype.constructor = Scene_ExOriginalSelectWindow;

	Scene_ExOriginalSelectWindow.prototype.create = function() {
        Scene_MenuBase.prototype.create.call(this);
		this.createHelpWindow();
        this.createChoiceWindow();
		this.reservePictures();
    };

	Scene_ExOriginalSelectWindow.prototype.createBackground = function() {
		if (backgroundImage !== "") {
			this._backgroundSprite = new Sprite();
			this._backgroundSprite.bitmap = ImageManager.loadPicture(backgroundImage);
			this.addChild(this._backgroundSprite);
		} else {
			Scene_MenuBase.prototype.createBackground.call(this);
		}

		this.createHelpImage();
	};

	Scene_ExOriginalSelectWindow.prototype.update = function() {
        Scene_MenuBase.prototype.update.call(this);
		this.setHelpText();
		this.setHelpImage();
    };

	Scene_ExOriginalSelectWindow.prototype.createChoiceWindow = function() {
		const rect = this.commandWindowRect();
        this._commandWindow = new Window_ExMenuCommand(rect);
        this._commandWindow.setHandler("ok", this.popScene.bind(this));
        this._commandWindow.setHandler("cancel", this.popScene.bind(this));
        this.addWindow(this._commandWindow);
    };

	Scene_ExOriginalSelectWindow.prototype.commandWindowRect = function() {
		const ww = commandWindowWidth;
		const wh = commandWindowHeight;
		const wx = commandWindowX;
		const wy = commandWindowY;
		return new Rectangle(wx, wy, ww, wh);
	};

	Scene_ExOriginalSelectWindow.prototype.helpWindowRect = function() {
		const ww = helpWindowWidth;
		const wh = helpWindowHeight;
		const wx = helpWindowX;
		const wy = helpWindowY;
		return new Rectangle(wx, wy, ww, wh);
	};

	Scene_ExOriginalSelectWindow.prototype.setHelpText = function() {
		const index = this._commandWindow.currentSymbol();
		const choiceList = choiceLists[index];

		if (choiceList) {
			if (!choiceList.switchId || $gameSwitches.value(choiceList.switchId)) {
				this._helpWindow.setText(choiceList.helptext);
				return;
			}
		}

		this._helpWindow.setText("");
    };

	Scene_ExOriginalSelectWindow.prototype.createHelpImage = function() {
		this._helpImage = new Sprite();
		this.addChild(this._helpImage);
	};

	Scene_ExOriginalSelectWindow.prototype.setHelpImage = function() {
		const index = this._commandWindow.currentSymbol();
		const choiceList = choiceLists[index];

		if (choiceList) {
			if (!choiceList.switchId || $gameSwitches.value(choiceList.switchId)) {
				this._helpImage.bitmap = ImageManager.loadPicture(choiceList.helpImage ?? "");
				this._helpImage.x = choiceList.helpImageX;
				this._helpImage.y = choiceList.helpImageY;
				return;
			}
		}

		this._helpImage.bitmap = null;
		this._helpImage.x = 0;
		this._helpImage.y = 0;
    };

	Scene_ExOriginalSelectWindow.prototype.needsCancelButton = function() {
		return cancelEnabled;
	};

	Scene_ExOriginalSelectWindow.prototype.reservePictures = function() {
		choiceLists.forEach((elm) => {
			if (elm.helpImage) {
				ImageManager.loadPicture(elm.helpImage);
			}
		});
	};


	//-----------------------------------------------------------------------------
	// Window_ExMenuCommand
	//
	// オリジナル選択ウィンドウを定義します。

	function Window_ExMenuCommand() {
		this.initialize(...arguments);
	}

	Window_ExMenuCommand.prototype = Object.create(Window_Command.prototype);
	Window_ExMenuCommand.prototype.constructor = Window_ExMenuCommand;

	const _Window_ExMenuCommand_initialize = Window_ExMenuCommand.prototype.initialize;
	Window_ExMenuCommand.prototype.initialize = function() {
		_Window_ExMenuCommand_initialize.apply(this, arguments);
        this.setSymbol();
    };

	Window_ExMenuCommand.prototype.makeCommandList = function() {
		choiceLists.forEach((elm, index) => {
			if (!elm.switchId || $gameSwitches.value(elm.switchId)) {
				this.addCommand(elm.label, index, true);
			} else if (elm.disabledLabel) {
				this.addCommand(elm.disabledLabel, index, false);
			}
		});
	};

	Window_ExMenuCommand.prototype.isCancelEnabled = function() {
		return cancelEnabled;
	};

	Window_ExMenuCommand.prototype.processOk = function() {
        $gameVariables.setValue(variableId, this.currentSymbol() + 1);
        Window_Command.prototype.processOk.call(this);
    };

    Window_ExMenuCommand.prototype.processCancel = function() {
        $gameVariables.setValue(variableId, -1);
        Window_Command.prototype.processCancel.call(this);
    };

	Window_ExMenuCommand.prototype.setSymbol = function() {
        if (iniPosition > 0) {
            this.selectSymbol(iniPosition - 1);
        } else {
            this.deselect();
        }
    };
})();
