//=============================================================================
// RPGツクールMZ - LL_StandingPictureMap.js v1.0.1
//-----------------------------------------------------------------------------
// ルルの教会 (Lulu's Church)
// https://nine-yusha.com/
//
// Secondary use of this plugin is not permitted.
//=============================================================================

/*:
 * @target MZ
 * @plugindesc マップ画面にアクター立ち絵を常時表示します。
 * @author ルルの教会
 * @url https://nine-yusha.com/plugin/
 *
 * @help LL_StandingPictureMap.js
 *
 * マップ画面にアクター立ち絵を常時表示します。
 * 立ち絵はステート状態や残りHPなどの条件に合わせて
 * リアルタイムで変化させることができます。
 *
 * 立ち絵の操作はプラグインコマンドでおこないます。
 *
 * プラグインコマンド:
 *   立ち絵の表示:         指定したアクターの立ち絵を表示します。
 *   モーションを再生:     立ち絵モーションを再生します。
 *   立ち絵の消去:         立ち絵を消去します。
 *   立ち絵表示のON・OFF:  立ち絵の表示／非表示を変更します。
 *   立ち絵の色調変更:     立ち絵の色調を変更します。
 *   制御文字の実行:       従来の制御文字で立ち絵を操作します。
 *
 * 下記のようにステート、スイッチ、変数条件で表示する立ち絵を複数定義できます。
 *   ・スイッチ1がONかつ毒状態の立ち絵
 *   ・変数1が10以上かつ毒状態の立ち絵
 *   ・スイッチ1がONの時の立ち絵
 *   ・毒状態の立ち絵
 *   ・スイッチ・ステート・変数条件なしの通常立ち絵 (最低限必要)
 *
 * 残りHP％で立ち絵を切り替える:
 *   まず「残りHP％」を「100」に設定した立ち絵リストを作成します。
 *   上記をコピーして「残りHP％」を「50」に変更し、立ち絵リストを複製します。
 *   これでHPが半分以下になった場合、「50」に設定した立ち絵が呼ばれます。
 *   残りHP％毎に、複数立ち絵を定義することも可能です。
 *
 * 画像ファイルの表示優先順:
 *   1. ステートID、スイッチID、変数条件全てに一致するもの
 *   2. ステートID、スイッチID両方に一致するもの
 *   3. ステートID、変数条件両方に一致するもの
 *   4. ステートIDのみ一致するもの
 *   5. スイッチID、変数条件両方に一致するもの
 *   6. スイッチIDのみ一致するもの
 *   7. 変数条件のみ一致するもの
 *   8. 条件なし (ステートID、スイッチID、変数条件全て設定なし)
 *   (上記の中で、残りHP％が最も低いものが優先して表示されます)
 *
 * 瞬きアニメーション:
 *   瞬き画像ファイルを設定すると、瞬きアニメーションが自動再生されます。
 *   (瞬きアニメーションを再生したくない場合は、画像を設定なしにしてください)
 *   ※再生順序: 画像1→画像2→なし (差分1枚で設定したい場合は画像1のみ設定)
 *
 * 立ち絵モーション一覧:
 *   yes(頷く)、yesyes(二回頷く)、no(横に揺れる)、noslow(ゆっくり横に揺れる)
 *   jump(跳ねる)、jumpjump(二回跳ねる)、jumploop(跳ね続ける)
 *   shake(ガクガク)、shakeloop(ガクガクし続ける)
 *   runleft(画面左へ走り去る)、runright(画面右へ走り去る)
 *   noslowloop(横に揺れ続ける)、huwahuwa(ふわふわ)
 *
 * 利用規約:
 *   このプラグインの二次利用、二次配布は禁止させていただきます。
 *
 * 作者: ルルの教会
 * 作成日: 2022/3/6
 *
 * @command showPicture
 * @text 立ち絵の表示
 * @desc 指定したアクターの立ち絵を表示します。
 *
 * @arg actorId
 * @text アクターID
 * @desc 立ち絵を表示する対象アクターです。
 * @type actor
 *
 * @command motionPicture
 * @text モーションを再生
 * @desc 立ち絵モーションを再生します。
 *
 * @arg motion
 * @text モーション
 * @desc モーションの種類です。
 * @default none
 * @type select
 * @option なし
 * @value none
 * @option 頷く
 * @value yes
 * @option 二回頷く
 * @value yesyes
 * @option 横に揺れる
 * @value no
 * @option ゆっくり横に揺れる
 * @value noslow
 * @option 横に揺れる(ループ)
 * @value noslowloop
 * @option 跳ねる
 * @value jump
 * @option 二回跳ねる
 * @value jumpjump
 * @option 跳ねる(ループ)
 * @value jumploop
 * @option ガクガク
 * @value shake
 * @option ガクガク(ループ)
 * @value shakeloop
 * @option 画面左へ走り去る
 * @value runleft
 * @option 画面右へ走り去る
 * @value runright
 * @option ふわふわ(ループ)
 * @value huwahuwa
 * @option 息づかい風(ループ)
 * @value breathing
 * @option 息づかい風2(ループ)
 * @value breathing2
 * @option ダメージ
 * @value damage
 *
 * @command erasePicture
 * @text 立ち絵の消去
 * @desc 立ち絵を消去します。
 *
 * @command setEnabledPicture
 * @text 立ち絵表示のON・OFF
 * @desc 立ち絵の表示／非表示を変更します。
 *
 * @arg enabled
 * @text 表示状態
 * @desc OFFにすると立ち絵を一時的に非表示にします。
 * ONにすると立ち絵を再表示します。
 * @default true
 * @type boolean
 *
 * @command setTone
 * @text 立ち絵の色調変更
 * @desc 立ち絵の色調を変更します。
 *
 * @arg toneR
 * @text 赤
 * @desc 色調のR成分です。 (-255～255)
 * @default 0
 * @type number
 * @min -255
 * @max 255
 *
 * @arg toneG
 * @text 緑
 * @desc 色調のG成分です。 (-255～255)
 * @default 0
 * @type number
 * @min -255
 * @max 255
 *
 * @arg toneB
 * @text 青
 * @desc 色調のB成分です。 (-255～255)
 * @default 0
 * @type number
 * @min -255
 * @max 255
 *
 * @arg toneC
 * @text グレー
 * @desc グレースケールの強さです。 (0～255)
 * @default 0
 * @type number
 * @min 0
 * @max 255
 *
 * @command processChar
 * @text 制御文字の実行
 * @desc 立ち絵を制御文字で操作します。
 * (従来の立ち絵プラグインと同じ制御文字で操作)
 *
 * @arg text
 * @text 制御文字(改行入力可)
 * @desc アクターn番の立ち絵を表示: \F[n]
 * 立ち絵を消去: \F[none]  モーション再生: \M[yes]
 * @type multiline_string
 *
 * @param smPictures
 * @text 立ち絵リスト
 * @desc マップに表示する立ち絵を定義します。
 * @default []
 * @type struct<smPictures>[]
 *
 * @param blinkSettings
 * @text 瞬きアニメーション
 * @desc ※この項目は使用しません
 *
 * @param blinkInterval
 * @text 瞬き間隔
 * @desc 瞬きする間隔の平均フレーム数です。 (1/60秒、初期値: 180)
 * 例えば180に設定すると、平均3秒間に1回瞬きをおこないます。
 * @default 180
 * @min 1
 * @max 9999
 * @type number
 * @parent blinkSettings
 *
 * @param blinkWaitCount
 * @text 瞬き表示時間
 * @desc 瞬き時の表示時間です。 (1/60秒、初期値: 4)
 * 数値を大きくすると目を閉じる時間が長くなります。
 * @default 4
 * @min 1
 * @max 9999
 * @type number
 * @parent blinkSettings
 *
 * @param spmButtonSettings
 * @text 表示ON・OFFボタン
 * @desc ※この項目は使用しません
 *
 * @param spmButtonImage
 * @text ボタン画像ファイル名
 * @desc 表示ON・OFFボタンに使用する画像です。
 * なしにするとボタンは表示されません。
 * @dir img/system
 * @type file
 * @require 1
 * @parent spmButtonSettings
 *
 * @param spmButtonImageWidth
 * @text 横幅
 * @desc ボタン画像の横幅です。
 * @default 96
 * @min -9999
 * @max 9999
 * @type number
 * @parent spmButtonSettings
 *
 * @param spmButtonImageHeight
 * @text 高さ
 * @desc ボタン画像の高さです。
 * ON・OFF時の画像を縦に並べ、半分の高さを入力します。
 * @default 48
 * @min -9999
 * @max 9999
 * @type number
 * @parent spmButtonSettings
 *
 * @param spmButtonImageX
 * @text X座標
 * @desc ボタンの表示位置(X)です。
 * @default 640
 * @min -9999
 * @max 9999
 * @type number
 * @parent spmButtonSettings
 *
 * @param spmButtonImageY
 * @text Y座標
 * @desc ボタンの表示位置(Y)です。
 * @default 2
 * @min -9999
 * @max 9999
 * @type number
 * @parent spmButtonSettings
 *
 * @param spmButtonShortcutKey
 * @text ショートカットキー
 * @desc 表示ON・OFFボタンのショートカットキーです。
 * 半角英数字の小文字で1文字入力します。(Aならa)
 * @default
 * @type string
 * @parent spmButtonSettings
 *
 * @param spmButtonDisableSwitchId
 * @text 無効化スイッチID
 * @desc 指定スイッチONでボタンを無効（非表示）にできます。
 * ショートカットキーもあわせて無効化されます。
 * @type switch
 * @parent spmButtonSettings
 *
 * @param foreFront
 * @text ウィンドウの前面に表示
 * @desc メッセージウィンドウよりも前面に表示します。
 * @type boolean
 * @default false
 *
 * @param catheBootPicture
 * @text 立ち絵画像の事前読み込み
 * @desc アツマールなどブラウザプレイ時の読み込み待ちを解消します。
 * 画像数や回線速度により起動が遅くなる場合があります。
 * @default true
 * @type boolean
 */

/*~struct~smPictures:
 *
 * @param actorId
 * @text アクターID
 * @desc アクターIDです。立ち絵を定義するアクターを選択してください。
 * @type actor
 *
 * @param stateId
 * @text ステートID
 * @desc 特定ステートで立ち絵を変更したい場合に使用します。
 * 通常時の立ち絵は空白(なし)で設定してください。
 * @type state
 *
 * @param switchId
 * @text スイッチID
 * @desc スイッチONで立ち絵を変更したい場合に使用します。
 * 通常時の立ち絵は空白(なし)で設定してください。
 * @type switch
 *
 * @param variableId
 * @text 変数ID
 * @desc 変数で立ち絵を変更したい場合に使用します。
 * 通常時の立ち絵は空白(なし)で設定してください。
 * @type variable
 *
 * @param variableType
 * @text 変数条件
 * @desc 変数IDを指定した際の比較条件です。
 * @default equal
 * @type select
 * @option 一致する
 * @value equal
 * @option 以上
 * @value higher
 * @option 以下
 * @value lower
 *
 * @param variableValue
 * @text 変数比較数値
 * @desc 変数IDを指定した際の比較する数値です。
 * @default 0
 * @min -99999999
 * @max 99999999
 * @type number
 *
 * @param hpPercentage
 * @text 残りHP％
 * @desc 残りHP％で立ち絵を変更したい場合に使用します。
 * 通常時の立ち絵は100％で設定してください。
 * @default 100
 * @min 0
 * @max 100
 * @type number
 *
 * @param imageName
 * @text 画像ファイル名
 * @desc 立ち絵として表示する画像ファイルを選択してください。
 * @dir img/pictures
 * @type file
 * @require 1
 *
 * @param imageNameBlink
 * @text 瞬き画像ファイル名1
 * @desc 瞬き時の差分画像ファイルを選択してください。
 * 画像サイズは基本となる立ち絵画像と統一してください。
 * @dir img/pictures
 * @type file
 * @require 1
 *
 * @param imageNameBlink2
 * @text 瞬き画像ファイル名2
 * @desc 瞬き時の差分画像ファイル2を選択してください。
 * 画像サイズは基本となる立ち絵画像と統一してください。
 * @dir img/pictures
 * @type file
 * @require 1
 *
 * @param origin
 * @text 原点
 * @desc 立ち絵の原点です。
 * @default 0
 * @type select
 * @option 左上
 * @value 0
 * @option 中央
 * @value 1
 *
 * @param x
 * @text X座標
 * @desc 立ち絵の表示位置(X)です。
 * @default 464
 * @min -9999
 * @max 9999
 * @type number
 *
 * @param y
 * @text Y座標
 * @desc 立ち絵の表示位置(Y)です。
 * @default 96
 * @min -9999
 * @max 9999
 * @type number
 *
 * @param scaleX
 * @text X拡大率
 * @desc 立ち絵の拡大率(X)です。
 * @default 100
 * @min -2000
 * @max 2000
 * @type number
 *
 * @param scaleY
 * @text Y拡大率
 * @desc 立ち絵の拡大率(Y)です。
 * @default 100
 * @min -2000
 * @max 2000
 * @type number
 */

(() => {
	"use strict";
	const pluginName = "LL_StandingPictureMap";

	const parameters = PluginManager.parameters(pluginName);
	const smPictures = JSON.parse(parameters["smPictures"] || "null");
	// 瞬きアニメーションの設定
	const blinkInterval = Number(parameters["blinkInterval"] || 180);
	const blinkWaitCount = Number(parameters["blinkWaitCount"] || 4);
	// 表示ON・OFFボタンの設定
	const spmButtonImage = String(parameters["spmButtonImage"] || "");
	const spmButtonImageWidth = Number(parameters["spmButtonImageWidth"] || 0);
	const spmButtonImageHeight = Number(parameters["spmButtonImageHeight"] || 0);
	const spmButtonImageX = Number(parameters["spmButtonImageX"] || 0);
	const spmButtonImageY = Number(parameters["spmButtonImageY"] || 0);
	const spmButtonShortcutKey = String(parameters["spmButtonShortcutKey"] || "");
	const spmButtonDisableSwitchId = Number(parameters["spmButtonDisableSwitchId"] || 0);
	// ウィンドウの前面に表示
	const catheBootPicture = eval(parameters["catheBootPicture"] || "true");
	// 立ち絵画像事前読み込み
	const foreFront = eval(parameters["foreFront"] || "false");

	let smPictureLists = [];
	if (smPictures) {
		smPictures.forEach((elm) => {
			smPictureLists.push(JSON.parse(elm));
		});
	}


	// アニメーションフレーム数定義
	const ANIMATION_FRAME = {
		"yes":        24,
		"yesyes":     48,
		"no":         24,
		"noslow":     48,
		"jump":       24,
		"jumpjump":   48,
		"jumploop":   48,
		"shake":      1,
		"shakeloop":  1,
		"runleft":    1,
		"runright":   1,
		"noslowloop": 96,
		"breathing":  96,
		"breathing2": 96,
		"huwahuwa":   192,
		"damage":     1,
		"none":       0,
	};
	// ループ維持アニメーション定義
	const ANIMATION_LOOPS = [
		"jumploop",
		"shakeloop",
		"noslowloop",
		"breathing",
		"breathing2",
		"huwahuwa"
	];


	// 立ち絵の表示
	PluginManager.registerCommand(pluginName, "showPicture", args => {
		const actorId = Number(args.actorId);
		const text = "\\F[" + actorId + "]";
		ExStandingPictureMap.processChar(text);
	});

	// モーションを再生
	PluginManager.registerCommand(pluginName, "motionPicture", args => {
		const motion = String(args.motion);
		const text = "\\M[" + motion + "]";
		ExStandingPictureMap.processChar(text);
	});

	// 立ち絵の消去
	PluginManager.registerCommand(pluginName, "erasePicture", args => {
		const text = "\\F[none]";
		ExStandingPictureMap.processChar(text);
	});

	// 立ち絵表示のON・OFF
	PluginManager.registerCommand(pluginName, "setEnabledPicture", args => {
		const enabled = eval(args.enabled || "true");
		$gameSystem._spmHidden = !enabled;
	});

	// 色調変更
	PluginManager.registerCommand(pluginName, "setTone", args => {
		const pictureTone = [Number(args.toneR), Number(args.toneG), Number(args.toneB), Number(args.toneC)];
		$gameSystem._spmTone = pictureTone;
		ExStandingPictureMap.refRequest();
	});

	// 制御文字の実行
	PluginManager.registerCommand(pluginName, "processChar", args => {
		const text = args.text;
		ExStandingPictureMap.processChar(text);
	});


	//-----------------------------------------------------------------------------
	// 画像ファイル名を取得
	//-----------------------------------------------------------------------------
	const getImageName = (actor, pictureLists, itemId = null) => {
		// アクターのステート情報を取得
		let actorStates = [];
		if (actor) {
			actorStates = actor._states;

			// 死亡時に直前のステートIDで判定するか？
			// if (actorStates.indexOf(1) !== -1 && deathBeforeStates) {
			// 	actorStates = actor.beforeStates();
			// }
		}
		let specificPicture = null;

		// アクターIDが一致する立ち絵を検索
		pictureLists = pictureLists.filter(function(item) {
			if (Number(item.actorId) == actor._actorId) {
				return true;
			}
		});

		// ステートにかかっているか？
		if (actorStates.length) {
			// ステートID・スイッチID・変数IDが有効な立ち絵リストを検索
			specificPicture = pictureLists.filter(function(item) {
				if (item.variableId && Number(item.variableId) !== 0) {
					if (
						actorStates.indexOf(Number(item.stateId)) !== -1 &&
						$gameSwitches.value(Number(item.switchId)) &&
						(
							String(item.variableType) == "equal" && $gameVariables.value(Number(item.variableId)) == Number(item.variableValue) ||
							String(item.variableType) == "higher" && $gameVariables.value(Number(item.variableId)) >= Number(item.variableValue) ||
							String(item.variableType) == "lower" && $gameVariables.value(Number(item.variableId)) <= Number(item.variableValue)
						)
					) {
						return true;
					}
				}
			});
			if (specificPicture.length) return checkHpPercentage(actor, specificPicture);
			// ステートID・スイッチIDが有効な立ち絵リストを検索
			specificPicture = pictureLists.filter(function(item) {
				if (actorStates.indexOf(Number(item.stateId)) !== -1 && $gameSwitches.value(Number(item.switchId)) && (Number(item.variableId) === 0 || !item.variableId)) {
					return true;
				}
			});
			if (specificPicture.length) return checkHpPercentage(actor, specificPicture);
			// ステートID・変数IDが有効な立ち絵リストを検索
			specificPicture = pictureLists.filter(function(item) {
				if (item.variableId && Number(item.variableId) !== 0) {
					if (
						actorStates.indexOf(Number(item.stateId)) !== -1 &&
						(Number(item.switchId) === 0 || !item.switchId) &&
						(
							String(item.variableType) == "equal" && $gameVariables.value(Number(item.variableId)) == Number(item.variableValue) ||
							String(item.variableType) == "higher" && $gameVariables.value(Number(item.variableId)) >= Number(item.variableValue) ||
							String(item.variableType) == "lower" && $gameVariables.value(Number(item.variableId)) <= Number(item.variableValue)
						)
					) {
						return true;
					}
				}
			});
			if (specificPicture.length) return checkHpPercentage(actor, specificPicture);
			// ステートIDが有効な立ち絵リストを検索
			specificPicture = pictureLists.filter(function(item) {
				if (actorStates.indexOf(Number(item.stateId)) !== -1 && (Number(item.switchId) === 0 || !item.switchId) && (Number(item.variableId) === 0 || !item.variableId)) {
					return true;
				}
			});
			if (specificPicture.length) return checkHpPercentage(actor, specificPicture);
		}

		// スイッチID・変数IDが有効な立ち絵リストを検索
		specificPicture = pictureLists.filter(function(item) {
			if (item.variableId && Number(item.variableId) !== 0) {
				if (
					(Number(item.stateId) === 0 || !item.stateId) &&
					$gameSwitches.value(Number(item.switchId)) &&
					(
						String(item.variableType) == "equal" && $gameVariables.value(Number(item.variableId)) == Number(item.variableValue) ||
						String(item.variableType) == "higher" && $gameVariables.value(Number(item.variableId)) >= Number(item.variableValue) ||
						String(item.variableType) == "lower" && $gameVariables.value(Number(item.variableId)) <= Number(item.variableValue)
					)
				) {
					return true;
				}
			}
		});
		if (specificPicture.length) return checkHpPercentage(actor, specificPicture);
		// スイッチIDが有効な立ち絵リストを検索
		specificPicture = pictureLists.filter(function(item) {
			if ((Number(item.stateId) === 0 || !item.stateId) && $gameSwitches.value(Number(item.switchId)) && (Number(item.variableId) === 0 || !item.variableId)) {
				return true;
			}
		});
		if (specificPicture.length) return checkHpPercentage(actor, specificPicture);
		// 変数IDが有効な立ち絵リストを検索
		specificPicture = pictureLists.filter(function(item) {
			if (item.variableId && Number(item.variableId) !== 0) {
				if (
					(Number(item.stateId) === 0 || !item.stateId) &&
					(Number(item.switchId) === 0 || !item.switchId) &&
					(
						String(item.variableType) == "equal" && $gameVariables.value(Number(item.variableId)) == Number(item.variableValue) ||
						String(item.variableType) == "higher" && $gameVariables.value(Number(item.variableId)) >= Number(item.variableValue) ||
						String(item.variableType) == "lower" && $gameVariables.value(Number(item.variableId)) <= Number(item.variableValue)
					)
				) {
					return true;
				}
			}
		});
		if (specificPicture.length) return checkHpPercentage(actor, specificPicture);

		// 上記で見つからなかった場合、通常の立ち絵を検索
		let sbPicture = pictureLists.filter(function(item) {
			if ((Number(item.stateId) === 0 || !item.stateId) && (Number(item.switchId) === 0 || !item.switchId) && (Number(item.variableId) === 0 || !item.variableId)) return true;
		});
		if (sbPicture.length) return checkHpPercentage(actor, sbPicture);
	};

	const checkHpPercentage = (actor, pictureLists) => {
		// アクターの残HP％を取得
		let hpRate = getHpRate(actor);
		// 最もHP%が低い立ち絵を適用する
		let minHpRate = 100;
		let result = null;
		pictureLists.forEach(function(item) {
			if (hpRate <= Number(item.hpPercentage) && minHpRate >= Number(item.hpPercentage)) {
				result = item;
				minHpRate = Number(item.hpPercentage);
			} else if (!item.hpPercentage && minHpRate >= 100) {
				// プラグインパラメータが更新されていない場合、便宜的に100として扱う
				result = item;
				minHpRate = Number(item.hpPercentage);
			}
		});
		return result;
	}

	// 敵キャラのHPレートを取得
	const getHpRate = (actor) => {
		if (!actor) return 0;
		return actor.mhp > 0 ? actor.hp / actor.mhp * 100 : 0;
	}


	//-----------------------------------------------------------------------------
	// Game_System
	//
	// 立ち絵制御用の独自配列を追加定義します。

	const _Game_System_initialize = Game_System.prototype.initialize;
	Game_System.prototype.initialize = function() {
		_Game_System_initialize.apply(this, arguments);

		this._spmHidden = false;
		this._spmTone = [0, 0, 0, 0];
		this._spmPictureActorId = null;
		this._spmPictureMotion = "";
	};

	//-----------------------------------------------------------------------------
	// ExStandingPictureMap
	//
	// 立ち絵を表示する独自のクラスを追加定義します。

	class ExStandingPictureMap {

		static create (scene) {
			// 立ち絵1
			scene._spmSprite = new Sprite_LL_SPicture();
			scene.addChildAt(scene._spmSprite, scene.children.indexOf(foreFront ? scene._windowLayer : scene._spriteset) + 1);

			// 立ち絵画像を事前読み込み
			if (catheBootPicture) {
				smPictureLists.forEach(function(elm) {
					ImageManager.loadPicture(elm.imageName);
				});
			}

			this._refRequest = false;
		}

		static refRequest () {
			this._refRequest = true;
		}

		static update (scene) {
			// 立ち絵非表示
			scene._spmSprite.visible = !$gameSystem._spmHidden;

			// 立ち絵1ピクチャ作成
			if ($gameSystem._spmPictureActorId) {
				let smPicture = null;
				let targetActor = $gameActors.actor($gameSystem._spmPictureActorId);
				if (targetActor) {
					smPicture = getImageName(targetActor, smPictureLists);
				}

				if (scene._spmSprite._currentPicture !== smPicture || this._refRequest) {
					scene._spmSprite.setPicture(smPicture, $gameSystem._spmPictureMotion);

					// ループアニメーションじゃない場合、アニメーションをリセット
					if (!ANIMATION_LOOPS.includes($gameSystem._spmPictureMotion)) {
						$gameSystem._spmPictureMotion = "";
					}
					this._refRequest = false;
				}
			}
		}

		static processChar (text) {
			text = text.replace(/\\V\[(\d+)\]/gi, (_, p1) =>
				$gameVariables.value(parseInt(p1))
			);

			// 専用制御文字を取得 (\F[s])
			let sPictureNumber = null;
			let processEscapeNumber = text.match(/\\F\[(\w+)\]/);
			if (processEscapeNumber) {
				if (processEscapeNumber[1]) {
					sPictureNumber = processEscapeNumber[1];
				}
			}
			// 専用制御文字を取得 (\M[s])
			let sPictureMotion = null;
			processEscapeNumber = text.match(/\\M\[(\w+)\]/);
			if (processEscapeNumber) {
				if (processEscapeNumber[1]) {
					sPictureMotion = processEscapeNumber[1];
				}
			}
			// 立ち絵1を更新
			if (sPictureNumber) {
				$gameSystem._spmPictureActorId = sPictureNumber;
			} else if (sPictureNumber === "none") {
				$gameSystem._spmPictureActorId = null;
			}
			if (sPictureMotion) {
				$gameSystem._spmPictureMotion = sPictureMotion;
			}

			this._refRequest = true;
		}
	}

	const _SceneManager_onKeyDown = SceneManager.onKeyDown;
	SceneManager.onKeyDown = function(event) {
		_SceneManager_onKeyDown.apply(this, arguments);

		// 無効化判定
		if ($gameSwitches.value(spmButtonDisableSwitchId)) {
			return;
		}
		// ショートカットキー入力判定
		if (event.key === spmButtonShortcutKey && this._scene.constructor === Scene_Map) {
			$gameSystem._spmHidden = !$gameSystem._spmHidden;
		}
	};

	//-----------------------------------------------------------------------------
	// Scene_Map
	//

	const _Scene_Map_update = Scene_Map.prototype.update;
	Scene_Map.prototype.update = function() {
		_Scene_Map_update.apply(this, arguments);
		ExStandingPictureMap.update(this);
	};

	const _Scene_Map_createDisplayObjects = Scene_Map.prototype.createDisplayObjects;
	Scene_Map.prototype.createDisplayObjects = function() {
		_Scene_Map_createDisplayObjects.apply(this, arguments);
		ExStandingPictureMap.create(this);
	};


	// 表示・非表示ボタン(タッチUI)
	const _Scene_Map_createButtons = Scene_Map.prototype.createButtons;
	Scene_Map.prototype.createButtons = function() {
		_Scene_Map_createButtons.apply(this, arguments);
		if (ConfigManager.touchUI) {
			this.createSPMButton();
		}
	};

	Scene_Map.prototype.createSPMButton = function() {
		this._menuButton2 = new Sprite_SPMButton();
		this._menuButton2.x = spmButtonImageX;
		this._menuButton2.y = spmButtonImageY;
		this._menuButton2.visible = false;
		this._menuButton2.setClickHandler(this.clickSPMButton.bind(this));
		this.addWindow(this._menuButton2);
	};

	Scene_Map.prototype.clickSPMButton = function() {
		// 無効化判定
		if ($gameSwitches.value(spmButtonDisableSwitchId)) {
			return;
		}
		$gameSystem._spmHidden = !$gameSystem._spmHidden;
	};

	const _Scene_Map_updateMenuButton = Scene_Map.prototype.updateMenuButton;
	Scene_Map.prototype.updateMenuButton = function() {
		_Scene_Map_updateMenuButton.apply(this, arguments);
		if (this._menuButton && this._menuButton2 && spmButtonImage !== "") {
			if (!$gameSwitches.value(spmButtonDisableSwitchId)) {
				this._menuButton2.visible = this._menuButton.visible;
			} else {
				this._menuButton2.visible = false;
			}
		}
	};

	const _Scene_Map_isAnyButtonPressed = Scene_Map.prototype.isAnyButtonPressed;
	Scene_Map.prototype.isAnyButtonPressed = function() {
		const menuButton = _Scene_Map_isAnyButtonPressed.call(this);
		return menuButton || (this._menuButton2 && this._menuButton2.isPressed());
	};

	//-----------------------------------------------------------------------------
	// Sprite_LL_SPicture
	//
	// 立ち絵を表示するための独自のスプライトを追加定義します。

	function Sprite_LL_SPicture() {
		this.initialize.apply(this, arguments);
	}

	Sprite_LL_SPicture.prototype = Object.create(Sprite.prototype);
	Sprite_LL_SPicture.prototype.constructor = Sprite_LL_SPicture;

	Sprite_LL_SPicture.prototype.initialize = function() {
		Sprite.prototype.initialize.call(this);

		this.bitmap = null;
		this.opacity = 255;
		this.opening = false;
		this.closing = false;
		this.originX = 0;
		this.originY = 0;

		this.setOverlayBitmap();
		this.initMembers();
	};

	Sprite_LL_SPicture.prototype.setOverlayBitmap = function() {
		this._spriteBlink = new Sprite();
		this._spriteBlink.visible = false;
		this.addChild(this._spriteBlink);
		this._spriteBlink2 = new Sprite();
		this._spriteBlink2.visible = false;
		this.addChild(this._spriteBlink2);
	};

	Sprite_LL_SPicture.prototype.initMembers = function() {
		this._blinkInterval = 0;
		this._blinkWait = 0;
		this._blinkAnimation = 0;

		this._animationCountLL = 0;
		this._motionLL = "";
		this._bitmapLoadedLL = false;

		this._currentPicture = null;
	};

	Sprite_LL_SPicture.prototype.update = function() {
		Sprite.prototype.update.call(this);

		this.updateFade();
		this.motionUpdate();
		this.updateBlink();
	};

	Sprite_LL_SPicture.prototype.updateBlink = function() {
		if (this.opening || this.closing) {
			this._spriteBlink.visible = false;
			this._spriteBlink2.visible = false;
			return;
		}

		if (this._blinkInterval <= 0) {
			this._blinkWait--;
			if (this._blinkWait <= 0) {
				this._blinkAnimation++;
				if (this._blinkAnimation > 2 || (this._blinkAnimation == 2 && !this._spriteBlink2.bitmap)) {
					this._blinkAnimation = 0;
				}

				switch (this._blinkAnimation) {
					case 1:
						this._spriteBlink.visible = true;
						this._spriteBlink2.visible = false;
						this._blinkWait = blinkWaitCount;
						this._blinkInterval = 0;
						break;
					case 2:
						this._spriteBlink.visible = false;
						this._spriteBlink2.visible = true;
						this._blinkWait = blinkWaitCount;
						this._blinkInterval = 0;
						break;
					default:
						this._spriteBlink.visible = false;
						this._spriteBlink2.visible = false;
						this._blinkWait = blinkWaitCount;
						this._blinkInterval = Math.randomInt(blinkInterval * 2);
						break;
				}
			}
		} else {
			this._blinkInterval--;
		}
	};

	Sprite_LL_SPicture.prototype.setPicture = function(sPicture, motion = "none") {
		if (!sPicture) {
			this._motionLL = "none";
			this._animationCountLL = 0;
			this._currentPicture = null;
			return;
		}

		// ベース画像
		this.bitmap = null;
		this.bitmap = ImageManager.loadPicture(sPicture.imageName);
		// 瞬き1
		this._spriteBlink.bitmap = null;
		this._spriteBlink.bitmap = ImageManager.loadPicture(sPicture.imageNameBlink);
		this._spriteBlink.visible = false;
		// 瞬き2
		this._spriteBlink2.bitmap = null;
		this._spriteBlink2.bitmap = ImageManager.loadPicture(sPicture.imageNameBlink2);
		this._spriteBlink2.visible = false;

		this._blinkInterval = Math.randomInt(blinkInterval * 2);
		this._blinkWait = blinkWaitCount;
		this._blinkAnimation = 0;
		// 画像が読み込まれたあとに実行
		this._bitmapLoadedLL = false;
		this.bitmap.addLoadListener(function() {
			this.x = Number(sPicture.x);
			this.y = Number(sPicture.y);
			// this.setBlendMode(Number(sPicture.blendMode));
			this.setColorTone($gameSystem._spmTone ? $gameSystem._spmTone : [0, 0, 0, 0]);
			// this.setBlendColor([0, 0, 0, 0]);
			this.scale.x = Number(sPicture.scaleX / 100);
			this.scale.y = Number(sPicture.scaleY / 100);
			this.anchor.x = Number(sPicture.origin) === 0 ? 0 : 0.5;
			this.anchor.y = Number(sPicture.origin) === 0 ? 0 : 0.5;
			this._bitmapLoadedLL = true;
		}.bind(this));

		this._motionLL = motion;
		this._animationCountLL = ANIMATION_FRAME[this._motionLL];
		this._currentPicture = sPicture;
	};

	Sprite_LL_SPicture.prototype.setBlendColor = function(color) {
		Sprite.prototype.setBlendColor.call(this, color);
		this._spriteBlink.setBlendColor(this.getBlendColor());
		this._spriteBlink2.setBlendColor(this.getBlendColor());
	};

	Sprite_LL_SPicture.prototype.setColorTone = function(tone) {
		Sprite.prototype.setColorTone.call(this, tone);
		this._spriteBlink.setColorTone(this.getColorTone());
		this._spriteBlink2.setColorTone(this.getColorTone());
	};

	Sprite_LL_SPicture.prototype.setBlendMode = function(mode) {
		this.blendMode = mode;
		this._spriteBlink.blendMode = mode;
		this._spriteBlink2.blendMode = mode;
	};

	Sprite_LL_SPicture.prototype.updateFade = function() {
		if (this._currentPicture) {
			if (!this._bitmapLoadedLL) return;
			if (!this._bitmapLoadedLL || this.opacity >= 255) {
				this.opening = false;
				return;
			}

			this.opening = true;
			this.closing = false;
			this.opacity += 255 / 15;
		} else {
			if (this.opacity === 0) {
				this.closing = false;
				return;
			}

			this.opening = false;
			this.closing = true;
			this.opacity -= 255 / 15;
		}
	}

	Sprite_LL_SPicture.prototype.motionUpdate = function() {
		if (
			this.opening ||
			this.closing ||
			this._animationCountLL < 1 ||
			!this._bitmapLoadedLL
		) return;

		if (this._motionLL == "yes") {
			if (this._animationCountLL > 12) {
				this.y += 2;
			} else {
				this.y -= 2;
			}
			this._animationCountLL -= 1;
		}
		if (this._motionLL == "yesyes") {
			if (this._animationCountLL > 36) {
				this.y += 2;
			} else if (this._animationCountLL > 24) {
				this.y -= 2;
			} else if (this._animationCountLL > 12) {
				this.y += 2;
			} else {
				this.y -= 2;
			}
			this._animationCountLL -= 1;
		}
		if (this._motionLL == "no") {
			if (this._animationCountLL > 18) {
				this.x += 2;
			} else if (this._animationCountLL > 6) {
				this.x -= 2;
			} else {
				this.x += 2;
			}
			this._animationCountLL -= 1;
		}
		if (this._motionLL == "noslow") {
			if (this._animationCountLL > 36) {
				this.x += 1;
			} else if (this._animationCountLL > 12) {
				this.x -= 1;
			} else {
				this.x += 1;
			}
			this._animationCountLL -= 1;
		}
		if (this._motionLL == "jump") {
			if (this._animationCountLL > 12) {
				this.y -= 2;
			} else {
				this.y += 2;
			}
			this._animationCountLL -= 1;
		}
		if (this._motionLL == "jumpjump") {
			if (this._animationCountLL > 36) {
				this.y -= 2;
			} else if (this._animationCountLL > 24) {
				this.y += 2;
			} else if (this._animationCountLL > 12) {
				this.y -= 2;
			} else {
				this.y += 2;
			}
			this._animationCountLL -= 1;
		}
		if (this._motionLL == "jumploop") {
			if (this._animationCountLL > 36) {
				this.y -= 2;
			} else if (this._animationCountLL > 24) {
				this.y += 2;
			}
			this._animationCountLL -= 1;
			if (this._animationCountLL == 0) this._animationCountLL = 48;
		}
		if (this._motionLL == "shake") {
			if (this._animationCountLL <= 2) {
				this.x -= 2;
				this._animationCountLL += 1;
			} else if (this._animationCountLL <= 4) {
				this.y -= 2;
				this._animationCountLL += 1;
			} else if (this._animationCountLL <= 6) {
				this.x += 4;
				this.y += 4;
				this._animationCountLL += 1;
			} else if (this._animationCountLL <= 8) {
				this.y -= 2;
				this._animationCountLL += 1;
			} else if (this._animationCountLL == 9) {
				this.x -= 2;
				this._animationCountLL += 1;
			} else if (this._animationCountLL == 10) {
				this.x -= 2;
				this._animationCountLL = 0;
			}
		}
		if (this._motionLL == "shakeloop") {
			if (this._animationCountLL <= 2) {
				this.x -= 1;
				this._animationCountLL += 1;
			} else if (this._animationCountLL <= 4) {
				this.y -= 1;
				this._animationCountLL += 1;
			} else if (this._animationCountLL <= 6) {
				this.x += 2;
				this.y += 2;
				this._animationCountLL += 1;
			} else if (this._animationCountLL <= 8) {
				this.y -= 1;
				this._animationCountLL += 1;
			} else if (this._animationCountLL <= 10) {
				this.x -= 1;
				this._animationCountLL += 1;
			}
			if (this._animationCountLL > 10) this._animationCountLL = 1;
		}
		if (this._motionLL == "runleft") {
			this.x -= 16;
			if (this.x < -2000) this._animationCountLL = 0;
		}
		if (this._motionLL == "runright") {
			this.x += 16;
			if (this.x > 2000) this._animationCountLL = 0;
		}
		if (this._motionLL == "noslowloop") {
			if (this._animationCountLL > 72) {
				this.x += 0.25;
			} else if (this._animationCountLL > 24) {
				this.x -= 0.25;
			} else {
				this.x += 0.25;
			}
			this._animationCountLL -= 1;
			if (this._animationCountLL == 0) this._animationCountLL = ANIMATION_FRAME["noslowloop"];
		}
		if (this._motionLL == "breathing") {
			if (this._animationCountLL > 72) {
				this.y += 0.5;
			} else if (this._animationCountLL > 48) {
				this.y -= 0.5;
			} else {
			}
			this._animationCountLL -= 1;
			if (this._animationCountLL == 0) this._animationCountLL = ANIMATION_FRAME["breathing"];
		}
		if (this._motionLL == "breathing2") {
			if (this._animationCountLL > 48) {
				this.y -= this.height * (this.anchor.y === 0 ? 0.0002 : 0.0001);
				this.scale.y += 0.0002;
			} else {
				this.y += this.height * (this.anchor.y === 0 ? 0.0002 : 0.0001);
				this.scale.y -= 0.0002;
			}
			this._animationCountLL -= 1;
			if (this._animationCountLL == 0) this._animationCountLL = ANIMATION_FRAME["breathing2"];
		}
		if (this._motionLL == "huwahuwa") {
			if (this._animationCountLL > 144) {
				this.y += 0.25;
			} else if (this._animationCountLL > 48) {
				this.y -= 0.25;
			} else {
				this.y += 0.25;
			}
			this._animationCountLL -= 1;
			if (this._animationCountLL == 0) this._animationCountLL = ANIMATION_FRAME["huwahuwa"];
		}
		if (this._motionLL == "damage") {
			if (this._animationCountLL <= 2) {
				this.x -= 4;
				this._animationCountLL += 1;
			} else if (this._animationCountLL <= 4) {
				this.y -= 4;
				this._animationCountLL += 1;
			} else if (this._animationCountLL <= 6) {
				this.x += 8;
				this.y += 8;
				this._animationCountLL += 1;
			} else if (this._animationCountLL <= 8) {
				this.y -= 4;
				this._animationCountLL += 1;
			} else if (this._animationCountLL == 9) {
				this.x -= 4;
				this._animationCountLL += 1;
			} else if (this._animationCountLL == 10) {
				this.x -= 4;
				this._animationCountLL = 0;
			}
		}
	};


	//-----------------------------------------------------------------------------
    // Sprite_SPMButton
    //
    // 立ち絵表示・非表示ボタンの独自Spriteを追加定義します。

    function Sprite_SPMButton() {
        this.initialize(...arguments);
    }

    Sprite_SPMButton.prototype = Object.create(Sprite_Clickable.prototype);
    Sprite_SPMButton.prototype.constructor = Sprite_SPMButton;

    Sprite_SPMButton.prototype.initialize = function(buttonType) {
        Sprite_Clickable.prototype.initialize.call(this);
        this._clickHandler = null;
        this._coldFrame = null;
        this._hotFrame = null;
        this.setupFrames();
    };

    Sprite_SPMButton.prototype.setupFrames = function() {
		const x = 0;
		const width = this.blockWidth();
		const height = this.blockHeight();
		this.loadButtonImage();
		this.setColdFrame(x, 0, width, height);
		this.setHotFrame(x, height, width, height);
		this.updateFrame();
		this.updateOpacity();
    };

    Sprite_SPMButton.prototype.blockWidth = function() {
        return spmButtonImageWidth;
    };

    Sprite_SPMButton.prototype.blockHeight = function() {
		return spmButtonImageHeight;
    };

    Sprite_SPMButton.prototype.loadButtonImage = function() {
        this.bitmap = ImageManager.loadSystem(spmButtonImage);
    };

    Sprite_SPMButton.prototype.update = function() {
        Sprite_Clickable.prototype.update.call(this);
        this.updateFrame();
        this.updateOpacity();
        this.processTouch();
    };

    Sprite_SPMButton.prototype.updateFrame = function() {
		const frame = $gameSystem._spmHidden ? this._hotFrame : this._coldFrame;
        if (frame) {
            this.setFrame(frame.x, frame.y, frame.width, frame.height);
        }
    };

    Sprite_SPMButton.prototype.updateOpacity = function() {
        // this.opacity = this._pressed ? 255 : 192;
		this.opacity = $gameSystem._spmHidden ? 255 : 192;
    };

    Sprite_SPMButton.prototype.setColdFrame = function(x, y, width, height) {
        this._coldFrame = new Rectangle(x, y, width, height);
    };

    Sprite_SPMButton.prototype.setHotFrame = function(x, y, width, height) {
        this._hotFrame = new Rectangle(x, y, width, height);
    };

    Sprite_SPMButton.prototype.setClickHandler = function(method) {
        this._clickHandler = method;
    };

    Sprite_SPMButton.prototype.onClick = function() {
        if (this._clickHandler) {
            this._clickHandler();
        }
    };
})();
